<?php

declare( strict_types=1 );

namespace ContentTranslation\Store;

use ContentTranslation\Entity\RecentSignificantEdit;
use ContentTranslation\LoadBalancer;
use Wikimedia\Rdbms\IDatabase;

class RecentSignificantEditStore {
	// Array of supported wiki families for the 'recent significant edit' entrypoint
	private const SUPPORTED_WIKI_FAMILIES = [
		'wikipedia',
	];
	private const TABLE_NAME = 'cx_significant_edits';

	/**
	 * The wiki family of the current wiki.
	 * @var string|null
	 */
	private $currentWikiFamily;

	/** @var LoadBalancer */
	private $lb;

	public function __construct( LoadBalancer $lb, ?string $currentWikiFamily ) {
		$this->lb = $lb;
		$this->currentWikiFamily = $currentWikiFamily;
	}

	public function isCurrentWikiFamilySupported(): bool {
		return (bool)array_search( $this->currentWikiFamily, self::SUPPORTED_WIKI_FAMILIES );
	}

	/**
	 * Given a RecentSignificantEdit instance, this method
	 * stores its fields, as a row inside the table.
	 * To make sure that at most 10 edits per user will exist,
	 * this method keeps only the 10 newest edits and deletes the rest.
	 *
	 * @param RecentSignificantEdit $edit
	 * @return int
	 */
	public function insert( RecentSignificantEdit $edit ): int {
		$values = $this->normalizeEdit( $edit );
		$primaryDb = $this->lb->getConnection( DB_PRIMARY );
		$values['cxse_timestamp'] = $primaryDb->timestamp();

		$primaryDb->insert( self::TABLE_NAME, $values, __METHOD__ );
		$newId = $primaryDb->insertId();
		$edit->setId( $newId );
		// Keep only the 10 newest edits and delete the rest
		$this->deleteOldEditsByUser( $edit->getUserId() );
		return $newId;
	}

	/**
	 * Given a RecentSignificantEdit instance with a valid id,
	 * this method updates the corresponding row inside the database,
	 * with the current values of the model and an updated edit
	 * timestamp.
	 *
	 * @param RecentSignificantEdit $edit
	 */
	public function update( RecentSignificantEdit $edit ): void {
		if ( $edit->getId() === null ) {
			return;
		}
		$primaryDb = $this->lb->getConnection( DB_PRIMARY );

		$values = $this->normalizeEdit( $edit );
		$values['cxse_timestamp'] = $primaryDb->timestamp();

		$primaryDb->update(
			self::TABLE_NAME,
			$values,
			[ 'cxse_id' => $edit->getId() ],
			__METHOD__
		);
	}

	/**
	 * Given a user id, this method deletes all the edits done by
	 * this user, that are older than the 10 first edits. This way
	 * we ensure that the "cx_significant_edits" table will not
	 * store more than 10 edits per user. If less than 10 edits exist,
	 * it returns without doing anything.
	 *
	 * @param int $userId
	 */
	private function deleteOldEditsByUser( int $userId ) {
		$edits = $this->findEditsByUser( $userId );
		if ( count( $edits ) <= 10 ) {
			return;
		}

		$editIdsToDelete = array_map( static function ( RecentSignificantEdit $edit ) {
			return $edit->getId();
		}, array_slice( $edits, 10 ) );

		$primaryDb = $this->lb->getConnection( DB_PRIMARY );
		$primaryDb->delete( self::TABLE_NAME, [ 'cxse_id' => $editIdsToDelete ], __METHOD__ );
	}

	/**
	 * Returns an array of RecentSignificantEdit instances, containing all
	 * the recent significant edits done by the given user and the current
	 * wiki family, ordered by their edit timestamp, in ascending order (i.e. oldest first).
	 *
	 * @param int $userId
	 * @return RecentSignificantEdit[]
	 */
	public function findEditsByUser( int $userId ): array {
		$conditions = [ 'cxse_global_user_id' => $userId, 'cxse_wiki_family' => $this->currentWikiFamily ];
		$options = [ 'ORDER BY' => 'cxse_timestamp asc' ];

		$replicaDb = $this->lb->getConnection( DB_REPLICA );
		$result = $replicaDb->select( self::TABLE_NAME, IDatabase::ALL_ROWS, $conditions, __METHOD__, $options );

		$edits = [];
		foreach ( $result as $row ) {
			$edits[] = $this->createEditFromRow( $row );
		}
		return $edits;
	}

	/**
	 * Given a user id, a page wikidata id and a language, this method
	 * searches for a recent significant edit, matching these arguments,
	 * an instance of RecentSignificantEdit class if exists.
	 *
	 * @param int $userId
	 * @param int $pageWikidataId
	 * @param string $language
	 * @return RecentSignificantEdit|null
	 */
	public function findExistingEdit( int $userId, int $pageWikidataId, string $language ): ?RecentSignificantEdit {
		$conditions = [
			'cxse_global_user_id' => $userId,
			"cxse_page_wikidata_id" => $pageWikidataId,
			"cxse_language" => $language,
			'cxse_wiki_family' => $this->currentWikiFamily
		];

		$replicaDb = $this->lb->getConnection( DB_REPLICA );
		$row = $replicaDb->selectRow( self::TABLE_NAME, IDatabase::ALL_ROWS, $conditions, __METHOD__ );

		if ( !$row ) {
			return null;
		}

		return $this->createEditFromRow( $row );
	}

	/**
	 * Given an stdClass instance, that represents
	 * a row from "cx_significant_edits" table, this
	 * method creates a RecentSignificantEdit object
	 * and returns it.
	 *
	 * @param \stdClass $row
	 * @return RecentSignificantEdit
	 */
	private function createEditFromRow( \stdClass $row ): RecentSignificantEdit {
		return new RecentSignificantEdit(
			(int)$row->cxse_id,
			(int)$row->cxse_global_user_id,
			(int)$row->cxse_page_wikidata_id,
			$row->cxse_language,
			$row->cxse_page_title,
			json_decode( $row->cxse_section_titles ),
			$row->cxse_timestamp
		);
	}

	/**
	 * Given a RecentSignificantEdit model, this method
	 * returns an array containing the corresponding
	 * fields and values, to be stored in the database.
	 *
	 * @param RecentSignificantEdit $edit
	 * @return array
	 */
	private function normalizeEdit( RecentSignificantEdit $edit ): array {
		return [
			'cxse_global_user_id' => $edit->getUserId(),
			'cxse_page_wikidata_id' => $edit->getPageWikidataId(),
			'cxse_language' => $edit->getLanguage(),
			'cxse_wiki_family' => $this->currentWikiFamily,
			'cxse_page_title' => $edit->getPageTitle(),
			'cxse_section_titles' => json_encode( $edit->getSectionTitles() ),
			'cxse_timestamp' => $edit->getTimestamp(),
		];
	}

}
